# -*- coding: utf-8 -*-

from bts.api import BTS
from bts.market import BTSMarket
import json
import os


def get_prefix(quote, base):
    prefix = "%s_%s" % (quote, base)
    prefix = prefix.replace('.', '-')
    return prefix


class BTSOrderBook(object):
    def __init__(self):
        self.market_list = [
            ["BOTSCNY", "BTS"], ["CNY", "BTS"], ["USD", "BTS"],
            ["GOLD", "BTS"], ["BTC", "BTS"], ["BDR.AAPL", "CNY"],
            ["NOTE", "BTS"]]
        self.init_done = False
        self.pusher = None
        self.order_book = {}
        self.deal_trx = {}
        for quote, base in self.market_list:
            prefix = get_prefix(quote, base)
            self.order_book[prefix] = {}
            self.deal_trx[prefix] = []
        self.init_market()
        self.execute()
        self.init_done = True

    def init_market(self):
        config_file = os.getenv("HOME") + "/.python-bts/bts_client.json"
        fd_config = open(config_file)
        config_bts = json.load(fd_config)["client_default"]
        fd_config.close()

        self.bts_client = BTS(config_bts["user"], config_bts["password"],
                              config_bts["host"], config_bts["port"])
        self.market = BTSMarket(self.bts_client)
        client_info = self.bts_client.get_info()
        self.height = int(client_info["blockchain_head_block_num"]) - 180

    def myPublish(self, topic, event):
        if self.pusher and self.init_done:
            self.pusher.emit(topic, event, namespace="")

    def publish_deal_trx(self, deal_trx):
        for trx in deal_trx:
            if trx["type"] == "bid":
                deal_type = "buy"
            else:
                deal_type = "sell"
            prefix = get_prefix(trx["quote"], trx["base"])
            format_trx = [prefix, trx["block"], trx["timestamp"],
                          deal_type, trx["price"], trx["volume"]]
            self.myPublish(
                u'bts.orderbook.%s.trx' % (format_trx[0]), format_trx[1:])
            self.myPublish(u'bts.orderbook.trx', format_trx)
            market = format_trx[0]
            if market not in self.deal_trx:
                self.deal_trx[market] = []
            self.deal_trx[market].append(format_trx[1:])
            print(format_trx)

    def publish_place_trx(self, place_trx):
        trx_id = ""
        for trx in place_trx:
            if trx_id == trx["trx_id"]:
                continue
            prefix = get_prefix(trx["quote"], trx["base"])
            trx_id = trx["trx_id"]
            if trx["cancel"]:
                trx["type"] = "cancel " + trx["type"]
            format_trx = [prefix, trx["block"], trx["timestamp"],
                          trx["type"], trx["price"], trx["amount"]]
            self.myPublish(
                u'bts.orderbook.%s.order' % (format_trx[0]), format_trx[1:])
            self.myPublish(u'bts.orderbook.order', format_trx)
            print(format_trx)

    def publish_order_book(self):
        for quote, base in self.market_list:
            prefix = get_prefix(quote, base)
            order_book = self.market.get_order_book(
                quote, base)
            order_book["bids"] = order_book["bids"][:10]
            order_book["asks"] = order_book["asks"][:10]
            if (prefix not in self.order_book or
               self.order_book[prefix] != order_book):
                self.order_book[prefix] = order_book
                self.myPublish(
                    u'bts.orderbook.%s' % prefix, order_book)

    def execute(self):
        client_info = self.bts_client.get_info()
        height_now = int(client_info["blockchain_head_block_num"])
        if(self.height < height_now):
            time_stamp = client_info["blockchain_head_block_timestamp"]
            self.myPublish(u'bts.blockchain.info',
                           {"height": height_now, "time_stamp": time_stamp})
            self.publish_order_book()
        while self.height < height_now:
            self.height += 1
            trxs = self.bts_client.get_block_transactions(
                self.height)
            recs = self.market.get_order_deal_rec(self.height)
            self.publish_deal_trx(recs)
            recs = self.market.get_order_place_rec(trxs)
            self.publish_place_trx(recs)
            self.market.update_order_owner(recs)



from gevent import monkey
monkey.patch_all()

#import datetime
import time
from threading import Thread
#from flask import Flask, render_template, session, request
from flask import Flask, render_template
from flask.ext.socketio import SocketIO, emit
from bts_wallet import BTSWallet
from bts_orderbook import BTSOrderBook

app = Flask(__name__)
app.debug = True
app.config['SECRET_KEY'] = 'secret!'
socketio = SocketIO(app)
bts_orderbook = BTSOrderBook()
bts_orderbook.pusher = socketio
bts_wallet = BTSWallet(bts_orderbook.bts_client)
bts_wallet.pusher = socketio


def background_thread():
    """Example of how to send server generated events to clients."""
    while True:
        time.sleep(10)
        bts_wallet.execute()
        bts_orderbook.execute()


@app.template_filter()
def datetimefilter(value, format='%Y/%m/%d %H:%M'):
    """convert a datetime to a different format."""
    return value.strftime(format)

app.jinja_env.filters['datetimefilter'] = datetimefilter


@app.route('/')
def index():
    current_height = bts_orderbook.height
    #current_time = datetime.datetime.now()
    return render_template(
        'index.html', title="BTS wallet", current_height=current_height)


@app.route('/wallet')
def wallet():
    current_height = bts_orderbook.height
    balance = bts_wallet.balance
    return render_template(
        'wallet.html', title="BTS wallet", current_height=current_height,
        balance=balance
    )


@app.route('/market')
def market():
    current_height = bts_orderbook.height
    order_book = bts_orderbook.order_book["CNY_BTS"]
    deal_trx = bts_orderbook.deal_trx["CNY_BTS"]
    return render_template(
        'market.html', title="BTS market", current_height=current_height,
        order_book=order_book, deal_trx=deal_trx
    )


@socketio.on('connect', namespace='')
def test_connect():
    emit('my response', {'data': 'Connected', 'count': 0})


def main():
    thread = Thread(target=background_thread)
    thread.start()

    socketio.run(app, use_reloader=False)

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

#from bts.api import BTS
#import json
#import os


class BTSWallet(object):
    def __init__(self, client):
        self.bts_client = client
        self.pusher = None
        self.balance = self.get_balance()
        client_info = self.bts_client.get_info()
        self.height = int(client_info["blockchain_head_block_num"])
        self.trx = self.get_trx()

    def myPublish(self, topic, event):
        if self.pusher:
            self.pusher.emit(topic, event, namespace="")

    def get_balance(self):
        return self.bts_client.get_balance()

    def get_account(self):
        return "baozi"

    def get_trx(self, start=0, end=-1):
        pass
        #return self.bts_client.get_trx()

    def execute(self):
        client_info = self.bts_client.get_info()
        height_now = int(client_info["blockchain_head_block_num"])
        if(height_now <= self.height):
            return
        balance = self.get_balance()
        if balance != self.balance:
            self.myPublish("balance", balance)
            self.balance = balance
        trx = self.get_trx(self.height+1, height_now)
        if trx is not None:
            self.trx.extend(trx)
            self.myPublish("trx", trx)
        self.height = height_now



# -*- coding: utf-8 -*-
"""Project metadata

Information describing the project.
"""

# The package name, which is also the "UNIX name" for the project.
package = 'btsbots-wallet'
project = "web wallet for BitShares"
project_no_spaces = project.replace(' ', '')
version = '0.1.2'
description = 'web wallet for BitShares'
authors = ['Alt']
authors_string = ', '.join(authors)
emails = ['pch957@163.com']
license = 'PRIVATE'
copyright = '2015 ' + authors_string
url = 'https://btsbots.com/'
